/** We have 3 CPUs in use. 
1st CPU is placed outside the house on the door.
2nd CPU is used to detected someone at the door (after unnecessary few steps, like domino falling).
3rd CPU is placed inside the house with the house owner. 
In our explanation we will refer to them as 1st, 2nd and 3rd CPUs. 

Steps:
1.  Someone pulls a string by the door. The 2nd CPU senses light change and sends a message to 1st CPU using MQTT (through 
    adafruit.io) that someone is at the door. 
    How is the message sent? 
    2nd CPU publishes a light value to a light feed which 1st CPU is subscribed to.
2.  1st CPU changes the value of virtual pin VO to 2 on Blynk. Using a Blynk webhook, the change of V0 value to 2 triggers a 
    make.com scenario.
3.  A line is added to google sheets with time of the new visits and adds 1 to a counter counting all visits. 
4.  1st CPU lets 3rd CPU know someone is at the door using Blynk and make.com. 
    How? 
    The new counter value is retrieved from google sheets, then it is send using an HTTP post request to the address of 
    the 3rd CPU webhook.
5.  Using Make.com a random happy gif is added to a Facebook post on the house owner’s Facebook page, posting
    “YAY! I have a new visitor” with the chosen gif. 
6.  The HTTP request (step 4) using Blynk webhook, triggers an alarm noise on 3rd CPU. The noise alerts the house owner 
    someone is at the door.
7.  On the house owner blink’s app it says “Wow, u have had X visitors this month!” (and avg calc? maybe add later)
8.  When it is 7pm virtual pin VO is changed to 1 (1st CPU code). Then, using make.com scenario and router we ask 
    chat gpt to come up with a funny sentence in Spanish to cheer up the person. 
    Then using a TextCortexAI translator (free version of google translate), the sentence is translated to English.
9.  The sentence is sent to the house owner’s email account through gmail (using Make.com), saying 
    “Hey Buddy, I know it has been a long day but I just wanted to say: {the translated sentence} :)”. 
10. 3rd CPU senses temperature in the house and sends it to the 1st CPU. How? 
3rd CPU sets virtual pin V2 value to the detected temperature. The 1st CPU is updated on V2 value using a webhook 
    (V2 callback function is on the 1st CPU code).
11. If temp is below 16 Celsius degrees, the house owner’s blynk app says “Don't forget a jacket!”, 
    if it’s between 16-30 it says “Weather is wonderful inside!” and if it’s above 30 it says “It's boiling inside. T shirts only!”. 
12. If house owner presses right button on 3rd CPU, 1st CPU shows green lights and says “please go in”. 
    Else, house owner presses left button on 3rd CPU 1st CPU shows red lights and says “sorry you can not go in”. 
    how does it work? 
    By changing value of virtual pin V1 on 3rd CPU code, the new value is updated on 3rd CPU (V1 callback function is on 1st CPU code). 
    If V1==1 entrance allowed, if V1==0 entrance denied. 

This code is the 1st CPU code.

  The circuit:
    No components attached to input/output.
    Sensors used- light.
    Uses the Blynk app.

  Video link: https://youtube.com/shorts/ifP_p79YtuQ
  Created By: Shira Cohen 318971827

**/


// Set ups and initializations for Blynk and mqtt
#define BLYNK_TEMPLATE_ID "TMPL66oQsnZzg"
#define BLYNK_TEMPLATE_NAME "Final Project"
#define BLYNK_AUTH_TOKEN "_MejWlcIjTnmZMw_z9FE-bGhLW_m0sqw"
#define BLYNK_PRINT SerialUSB

// Importing necesary libraries
#include <ESP8266_Lib.h>
#include <BlynkSimpleShieldEsp8266.h>
#include <Adafruit_CircuitPlayground.h>
#include <TimeLib.h>
#include "Talkie.h"
#include "Vocab_US_Large.h"
#include "Vocab_US_TI99.h"
#include <Arduino.h>
#include <Adafruit_CircuitPlayground.h>
#define TINY_GSM_MODEM_ESP8266

#include <TinyGsmClient.h>
#include <PubSubClient.h>
#define ESP8266_BAUD 115200

#define SerialMon Serial
#define SerialAT Serial1

#define MQTT_USERNAME "sshiracccc"
#define MQTT_PASSWORD "aio_zgZX127Kfh0HQEXUd1lWoSO9XO5q"

// My WiFi credentials 
char ssid[] = "wireless";
char pass[] = "5101051010";

// Used for mqtt set up
const char MQTT_CLIENTID[] = MQTT_USERNAME __DATE__ __TIME__;
const char* broker = "io.adafruit.com";

// The feed we use to retrieve data from
const char lightTopic[] = MQTT_USERNAME "/feeds/light";

// More mqtt set ups
TinyGsm modem(SerialAT);
TinyGsmClient client(modem);
PubSubClient mqtt(client);


ESP8266 wifi(&SerialAT);
uint32_t lastReconnectAttempt = 0;

// variables initializaions

// used to retrieve data from feeds
char temp[50];
String tempStr;
long lastPublished = 0;

char light[50];
String lightStr;
long lastLightPublished = 0;

// used to say message (out loud) to passenger 
Talkie voice;

// used to make sure happy message is only sent once
float X,Y,Z;
boolean runned = false; 
boolean visitsToday = false;
boolean emailedNoVisits = false;
 
void setup() {
  
  CircuitPlayground.begin();
  SerialUSB.begin(115200);
  SerialAT.begin(ESP8266_BAUD);
  delay(10);

  // Blynk set up
  Blynk.begin(BLYNK_AUTH_TOKEN, wifi, ssid, pass);
  setTime(17, 29, 0, 22, 07, 2023);        // Set the initial time (hour, minute, second, day, month, year)
  int tempC = CircuitPlayground.temperature();
  Blynk.virtualWrite(V2, tempC);
  Blynk.virtualWrite(V3, "Waiting for confirmation to go inside...");

  // mqtt set up
  delay(1000);
  SerialAT.begin(115200);
  SerialMon.println("Initializing modem...");
  modem.restart();
  String modemInfo = modem.getModemInfo();
  SerialMon.print("Modem Info: ");
  SerialMon.println(modemInfo);
  SerialMon.print(F("Setting SSID/password..."));
  if (!modem.networkConnect(ssid, pass)) {
    SerialMon.println(" fail");
    while(true);
  }
  SerialMon.println(" success");
  SerialMon.print("Waiting for network...");
  if (!modem.waitForNetwork()) {
    SerialMon.println(" fail");
    while(true);
  }
  SerialMon.println(" success");
  if (modem.isNetworkConnected()) {
    SerialMon.println("Network connected");
  }
  mqtt.setServer(broker, 1883);  
  mqtt.setCallback(mqttCallback); 
  
}

// function that changes the leds' colours on the CPU
void lightUpLEDs(int redValue, int greenValue, int blueValue) {
  for (int i = 0; i < 10; i++) {
    CircuitPlayground.setPixelColor(i, redValue, greenValue, blueValue);
  } 
}

void loop() {

  // Extracting the individual components of the time
  time_t timeNow = now();
  int currentHour = hour(timeNow);
  int currentMinute = minute(timeNow);

  // If the time is 7pm - steps 7,8 are implemented  
  if (currentHour==19 && currentMinute == 0 && !emailedNoVisits) {
    emailedNoVisits = true;
    // change of V0 value to 1 triggers a router scenario in make.com
    Blynk.virtualWrite(V0, 1);
  }

  // Makes sure the message is sent only once a day at 19:00
  if (currentHour==19 && currentMinute==1) {
    emailedNoVisits = false;
  }

  // mqtt initializions
  if (!mqtt.connected()) {
    uint32_t t = millis();
    if (t - lastReconnectAttempt > 10000L) {
      lastReconnectAttempt = t;
      if (mqttConnect()) {
        lastReconnectAttempt = 0;
      }
  }
    delay(100);
    return;
  }
  mqtt.loop();
  delay(1000);
}

// functions that connects mqtt
boolean mqttConnect() {
  SerialMon.print("Connecting to ");
  SerialMon.print(broker);
  // no password? no problem
  // boolean status = mqtt.connect(MQTT_CLIENTID);
  boolean status = mqtt.connect(MQTT_CLIENTID, MQTT_USERNAME, MQTT_PASSWORD);
  if (status == false) {
    SerialMon.println(" fail");
    return false;
  }
  SerialMon.println(" success");
  mqtt.subscribe(lightTopic); 
  return mqtt.connected();
}

// this function is called everytime the light value is changed on the light feed we are subcribed to
void mqttCallback(char* topic, byte* payload, unsigned int len) {

  // Only proceed if incoming message's topic matches
  if (String(topic) == lightTopic) {
    Serial.print("recieved a change in light");
    // By changing V0 value to 2, a senario is starting to run in make.com  (step 2 and forward)
    Blynk.virtualWrite(V0, 2);
  }
}


// This function will be called every time the botton in Blynk app writes values to the Virtual Pin V1.
// other 3rd CPU uses webhooks to change V1 value and trigger this function.
BLYNK_WRITE(V1) {
    
  String input = param.asStr();
  int allowedEntrance = 0;
  int temp = 0;

  // Parsing the string and extracting integers a and b
  int i = 0;
  while (input[i] != '\0' && input[i] != ':') {
    allowedEntrance = allowedEntrance * 10 + (input[i] - '0');
    i++;
  }
  i++; // Move past the colon character ':'
  while (input[i] != '\0') {
    temp = temp * 10 + (input[i] - '0');
    i++;
  }

  // lets guest know temperature inside by change V2 value (1st CPU uses webhook to detect change)
  Blynk.virtualWrite(V2, temp);
  if ( temp < 16 ) {
    Blynk.virtualWrite(V3, "Don't forget a jacket!");
  } else if ( temp >= 16 && temp <= 30 ) {
    Blynk.virtualWrite(V3, "Weather is wonderful inside!");
  } else {
    Blynk.virtualWrite(V3, "It's boiling inside. T shirts only!");
  }


  // entrance allowed
  if ( allowedEntrance == 1 ) {
    lightUpLEDs(0,255,0);
    voice.say(spt_PLEASE);
    voice.say(spt_GO);
    voice.say(spt_IN);
    delay(2000);
    lightUpLEDs(0,0,0);
  } 

  // entrance denied
  if ( allowedEntrance == 0 ) {
    lightUpLEDs(255,0,0);
    voice.say(spt_SORRY);
    voice.say(spt_YOU);
    voice.say(spt_CAN);
    voice.say(sp2_NOT);
    voice.say(sp2_GO);
    voice.say(spt_IN);
    delay(2000);
    lightUpLEDs(0,0,0);
  } 
}
